@@ -3,10 +3,14 @@ extern crate clap;
 
 use clap::{App, AppSettings, Arg, ArgMatches, SubCommand};
 use std::option::Option::Some;
+use std::path::PathBuf;
+use vfs::{PhysicalFS, VfsPath};
 
 use releaser::brew;
+use releaser::cargo::Cargo;
+use releaser::git::Git;
 use releaser::scoop;
-use releaser::workflow::{Crate, Release, Workspace};
+use releaser::workflow::{Crate, Releasable, Release, Workspace};
 use releaser::Increment;
 
 const PATH: &str = "PATH";
@@ -28,12 +32,12 @@ fn main() {
 fn workspace(cmd: &ArgMatches) {
     let delay_seconds = cmd.value_of("delay").unwrap_or("");
     let delay_seconds: u64 = delay_seconds.parse().unwrap_or(20);
-    let r = Workspace::new(delay_seconds);
+    let r = Workspace::new(delay_seconds, Cargo::default(), Git::default());
     release(cmd, r);
 }
 
 fn single_crate(cmd: &ArgMatches) {
-    let r = Crate::new();
+    let r = Crate::new(Cargo::default(), Git::default());
     release(cmd, r);
 }
 
@@ -47,6 +51,11 @@ fn brew(cmd: &ArgMatches) {
 
     let crate_path = cmd.value_of("crate").unwrap_or("");
     let base_uri = cmd.value_of("base").unwrap_or("");
+
+    let crate_path: VfsPath = PhysicalFS::new(PathBuf::from(crate_path)).into();
+    let linux_path: VfsPath = PhysicalFS::new(PathBuf::from(linux_path)).into();
+    let macos_path: VfsPath = PhysicalFS::new(PathBuf::from(macos_path)).into();
+
     let b = brew::new_brew(crate_path, linux_path, macos_path, base_uri);
     output_string(cmd, b)
 }
@@ -56,29 +65,45 @@ fn scoop(cmd: &ArgMatches) {
     let binary_path = cmd.value_of("binary").unwrap_or("");
     let crate_path = cmd.value_of("crate").unwrap_or("");
     let base_uri = cmd.value_of("base").unwrap_or("");
+
+    let crate_path: VfsPath = PhysicalFS::new(PathBuf::from(crate_path)).into();
+    let binary_path: VfsPath = PhysicalFS::new(PathBuf::from(binary_path)).into();
+
     let scoop = scoop::new_scoop(crate_path, binary_path, exe_name, base_uri);
     output_string(cmd, scoop)
 }
 
+enum ErrorCode {
+    NoOutputProduced = 1,
+    FileWriteError = 2,
+    ReleaseError = 3,
+}
+
 fn output_string(cmd: &ArgMatches, s: Option<String>) {
-    if let Some(b) = s {
-        let output_path = cmd.value_of("output");
-        match output_path {
-            None => println!("{}", b),
-            Some(path) => {
-                let result = std::fs::write(path, b);
-                match result {
-                    Ok(_) => {}
-                    Err(e) => println!("{}", e),
+    match s {
+        None => std::process::exit(ErrorCode::NoOutputProduced as i32),
+        Some(b) => {
+            let output_path = cmd.value_of("output");
+            match output_path {
+                None => println!("{}", b),
+                Some(path) => {
+                    let result = std::fs::write(path, b);
+                    match result {
+                        Ok(_) => {}
+                        Err(e) => {
+                            eprintln!("{}", e);
+                            std::process::exit(ErrorCode::FileWriteError as i32);
+                        }
+                    }
                 }
             }
         }
     }
 }
 
-fn release<R>(cmd: &ArgMatches, release: R)
+fn release<'a, R>(cmd: &'a ArgMatches, release: R)
 where
-    R: Release,
+    R: Release<'a>,
 {
     let path = cmd.value_of(PATH).unwrap();
     let incr = cmd.value_of(INCR).unwrap();
@@ -94,9 +119,14 @@ where
         return;
     }
 
-    match release.release(path, inc.unwrap()) {
+    let r: VfsPath = PhysicalFS::new(PathBuf::from(path)).into();
+    let root = Releasable::new(path, r);
+    match release.release(root, inc.unwrap()) {
         Ok(()) => {}
-        Err(e) => eprintln!("Error: {}", e),
+        Err(e) => {
+            eprintln!("Path:\t{}\nError:\t{}", path, e);
+            std::process::exit(ErrorCode::ReleaseError as i32);
+        }
     }
 }
 
